#version 330 core

in vec2 screen_ndc;

// updated rarely
uniform vec2 screen_resolution;
uniform vec2 tile_size;
uniform vec2 grid_count;
uniform sampler2D tile_sheet_texture;

// updated each frame
uniform usampler2D tile_id_texture;
uniform sampler2D tile_foreground_texture;
uniform sampler2D tile_background_texture;

out vec4 fragColor;

void main()
{
  vec2 frag_uv = (screen_ndc + 1.0) / 2.0;

  if (textureSize(tile_id_texture, 0) == vec2(grid_count)) {
    vec4 tile_foreground = texture(tile_foreground_texture, frag_uv);
    vec4 tile_background = texture(tile_background_texture, frag_uv);

    int tile_id = int(texture(tile_id_texture, frag_uv).r);
    int tile_texture_row = 15 - (tile_id / 16);
    int tile_texture_col = tile_id % 16;

    vec2 tile_texture_base_pixel = vec2(tile_texture_col, tile_texture_row) * tile_size;
    vec2 grid_pixel = frag_uv * grid_count;
    vec2 tile_texture_pixel_offset = (grid_pixel - floor(grid_pixel)) * tile_size;

    vec2 tile_texture_pixel = tile_texture_base_pixel + tile_texture_pixel_offset;
    vec2 tile_texture_uv = tile_texture_pixel / vec2(textureSize(tile_sheet_texture, 0));
    vec4 tile_id_color = texture(tile_sheet_texture, tile_texture_uv);

    // place the tile color over top the tile itself.
    vec4 overlay = mix(tile_id_color, tile_foreground, tile_foreground.a);
    // place that over top of the screen
    fragColor = mix(tile_background, overlay, tile_id_color.a);
  } else {
    fragColor = vec4(1.0, 0.0, 0.0, 1.0);
  }
}
